@@ -4,7 +4,7 @@
 #   Copyright (c) 2012 Red Hat, Inc. All rights reserved.
 #   Author: Petr Splichal <psplicha@redhat.com>
 #
-#   Copyright (c) 2018,2020-2021 Kiwi TCMS project. All rights reserved.
+#   Copyright (c) 2018,2020-2022 Kiwi TCMS project. All rights reserved.
 #   Author: Alexander Todorov <info@kiwitcms.org>
 #
 # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -23,7 +23,7 @@
 
 """
 This module provides a dictionary based Python interface for the
-Kiwi TCMS test case management system.
+Kiwi TCMS test management system. It operates via the XML-RPC protocol.
 
 Installation::
 
@@ -52,8 +52,8 @@ where ``EXAMPLE.COM`` matches the realm in your organization.
     ``C:\\Users\\tcms-bot\\.tcms.conf`` on Windows, where ``tcms-bot``
     is the username on the local computer.
 
-    It's also possible to provide system-wide config in ``/etc/tcms.conf``,
-    which is valid only on Linux!
+    It's also possible to provide system-wide config in ``/etc/tcms.conf``
+    on Linux and ``C:\\tcms.conf`` on Windows!
 
     Execute the following Python snippet to find the exact location on your
     system::
@@ -104,29 +104,28 @@ class TCMS:  # pylint: disable=too-few-public-methods
         # Try system settings when the config does not exist in user directory
         if not os.path.exists(self._path):
             self._path = "/etc/tcms.conf"
+        if not os.path.exists(self._path):
+            self._path = "c:/tcms.conf"
         if not os.path.exists(self._path):
             raise Exception(f"Config file '{self._path}' not found")
 
         config = ConfigParser()
         config.read(self._path)
 
-        # Make sure the server URL is set
-        try:
-            config['tcms']['url'] is not None
-        except (KeyError, AttributeError) as err:
-            raise Exception(f"No url found in {self._path}") from err
-
         if strtobool(config['tcms'].get('use_kerberos', 'False')):
             # use Kerberos
-            TCMS._connection = TCMSKerbXmlrpc(None, None,
-                                              config['tcms']['url']).server
+            TCMS._connection = TCMSKerbXmlrpc(
+                None, None, self.server_url(config)
+            ).server
             return
 
         try:
             # use password authentication
-            TCMS._connection = TCMSXmlrpc(config['tcms']['username'],
-                                          config['tcms']['password'],
-                                          config['tcms']['url']).server
+            TCMS._connection = TCMSXmlrpc(
+                config['tcms']['username'],
+                config['tcms']['password'],
+                self.server_url(config)
+            ).server
         except KeyError as err:
             raise Exception(
                 f"username/password required in {self._path}") from err
@@ -140,3 +139,15 @@ class TCMS:  # pylint: disable=too-few-public-methods
         you can call various server-side functions.
         """
         return TCMS._connection
+
+    def server_url(self, config):
+        """
+        Returns the server URL and performs various sanity checks!
+        """
+        # Make sure the server URL is set
+        try:
+            config['tcms']['url'] is not None
+        except (KeyError, AttributeError) as err:
+            raise Exception(f"No url found in {self._path}") from err
+
+        return config['tcms']['url'].replace('json-rpc', 'xml-rpc')
