@@ -1,13 +1,16 @@
-# Copyright (c) 2019-2021 Alexander Todorov <atodorov@MrSenko.com>
+# Copyright (c) 2019-2022 Alexander Todorov <atodorov@MrSenko.com>
 
 import os
 from datetime import datetime
+# change this to import cache in Python 3.9
+from functools import lru_cache as cache
 
 from . import TCMS
+from .version import __version__
 
 
-class Backend:  # pylint: disable=too-many-instance-attributes
-
+# pylint: disable=too-many-instance-attributes, too-many-public-methods
+class Backend:
     """
         Facilitates RPC communications with the backend and implements
         behavior described at:
@@ -38,16 +41,33 @@ class Backend:  # pylint: disable=too-many-instance-attributes
     """
 
     _statuses = {}
+    name = 'tcms_api.plugin_helpers.Backend'
+    version = __version__
 
-    def __init__(self, prefix=''):
+    @property
+    def prefix(self):
+        """
+            Prefix may be overriden via ``TCMS_PREFIX`` environment variable.
+
+            .. versionadded:: 11.2
+        """
+        return os.environ.get('TCMS_PREFIX', self._prefix)
+
+    def __init__(self, prefix='', verbose=False):
         """
             :param prefix: Prefix which will be added to TestPlan.name and
                            TestRun.summary
 
                            .. versionadded:: 5.2
             :type prefix: str
+            :param verbose: If ``True`` will print info about created records.
+                            Defaults to ``False``
+
+                           .. versionadded:: 11.3
+            :type verbose: bool
         """
-        self.prefix = prefix
+        self._prefix = prefix
+        self.verbose = verbose
 
         self.rpc = None
         self.run_id = None
@@ -57,6 +77,15 @@ class Backend:  # pylint: disable=too-many-instance-attributes
         self.priority_id = None
         self.confirmed_id = None
 
+    def log_info(self, was_created, obj_prefix, obj_id):
+        if self.verbose:
+            verb = {
+                True:  "added",
+                False: "reuse",
+            }[was_created]
+
+            print(f"{verb}: {obj_prefix}-{obj_id}")
+
     def configure(self):
         """
             This method is reading all the configs from the environment
@@ -301,27 +330,19 @@ class Backend:  # pylint: disable=too-many-instance-attributes
         """
         return os.environ.get('TCMS_PLAN_ID', 0)
 
+    @property
+    @cache(maxsize=128)
     def default_tester_id(self):
         """
             Used internally and by default this is the user sending the API
             request. Use `$TCMS_DEFAULT_TESTER_ID` to override!
 
-            .. warning::
-
-                Requires ``auth.view_user`` permission if not overriden via
-                environment variable! See
-                https://kiwitcms.readthedocs.io/en/latest/admin.html?highlight=auth.view_user#managing-permissions
-
             Plugins may want to override this.
 
             :return: User ID
             :rtype: int
         """
-        id_from_env = os.environ.get('TCMS_DEFAULT_TESTER_ID')
-        if id_from_env:
-            return id_from_env
-
-        return self.rpc.User.filter()[0]['id']
+        return os.environ.get('TCMS_DEFAULT_TESTER_ID')
 
     def get_plan_id(self, run_id):
         """
@@ -333,6 +354,11 @@ class Backend:  # pylint: disable=too-many-instance-attributes
 
                 For internal use by `.configure()`!
 
+            If ``TCMS_PARENT_PLAN`` environment variable is specified and a new
+            TestPlan is created then it will be created as a child TP.
+
+            .. versionadded:: 11.2
+
             :param run_id: ``tcms.testruns.models.TestRun`` PK
             :type run_id: int
             :return: ``tcms.testplans.models.TestPlan`` PK
@@ -340,6 +366,7 @@ class Backend:  # pylint: disable=too-many-instance-attributes
         """
         plan_id = self.external_plan_id()
         if plan_id:
+            self.log_info(False, "TP", plan_id)
             return plan_id
 
         result = self.rpc.TestRun.filter({'pk': run_id})
@@ -348,6 +375,7 @@ class Backend:  # pylint: disable=too-many-instance-attributes
             version_id, version_name = self.get_version_id(product_id)
 
             name = f'{self.prefix} Plan for {product_name} ({version_name})'
+            was_created = False
             result = self.rpc.TestPlan.filter({'name': name,
                                                'product': product_id,
                                                'product_version': version_id})
@@ -355,20 +383,31 @@ class Backend:  # pylint: disable=too-many-instance-attributes
             if not result:
                 plan_type_id = self.get_plan_type_id()
 
-                result = [self.rpc.TestPlan.create({
+                args = {
                     'name': name,
-                    'text': 'Created by tcms_api.plugin_helpers.Backend',
+                    'text': self.created_by_text,
                     'product': product_id,
                     'product_version': version_id,
                     'is_active': True,
                     'type': plan_type_id,
-                    'author': self.default_tester_id(),
-                })]
+                }
+
+                if self.default_tester_id:
+                    args['author'] = self.default_tester_id
+
+                parent_plan_id = os.environ.get('TCMS_PARENT_PLAN')
+                if parent_plan_id:
+                    args['parent'] = parent_plan_id
+
+                result = [self.rpc.TestPlan.create(args)]
+                was_created = True
 
             # newly created TP
+            self.log_info(was_created, "TP", result[0]['id'])
             return result[0]['id']
 
         # TP to which existing TR is assigned
+        self.log_info(False, "TP", result[0]['plan'])
         return result[0]['plan']
 
     def get_run_id(self):
@@ -385,6 +424,7 @@ class Backend:  # pylint: disable=too-many-instance-attributes
             :return: ``tcms.testruns.models.TestRun`` PK
             :rtype: int
         """
+        was_added = False
         run_id = os.environ.get('TCMS_RUN_ID')
 
         if not run_id:
@@ -393,22 +433,32 @@ class Backend:  # pylint: disable=too-many-instance-attributes
             build_id, build_number = self.get_build_id(version_id)
             plan_id = self.get_plan_id(0)
             # TR.manager is always the author of the TP, which is either
-            # another existing user (existing TP) or self.default_tester_id()
+            # another existing user (existing TP) or self.default_tester_id
             # in case of newly created TP
             manager_id = self.rpc.TestPlan.filter({
                 'pk': plan_id
             })[0]['author']
 
-            testrun = self.rpc.TestRun.create({
+            args = {
                 'summary': f'{self.prefix} Results for {product_name}, '
                            f'{version_val}, {build_number}',
                 'manager': manager_id,
-                'default_tester': self.default_tester_id(),
                 'plan': plan_id,
                 'build': build_id,
-            })
+                'start_date': datetime.now().isoformat().replace(
+                    'T', ' ')[:19],
+            }
+            if self.default_tester_id:
+                args['default_tester'] = self.default_tester_id
+            else:
+                # b/c TestRun.create() always requires this argument
+                args['default_tester'] = manager_id
+
+            testrun = self.rpc.TestRun.create(args)
+            was_added = True
             run_id = testrun['id']
 
+        self.log_info(was_added, "TR", run_id)
         return int(run_id)
 
     def finish_test_run(self):
@@ -454,7 +504,7 @@ class Backend:  # pylint: disable=too-many-instance-attributes
                 'category': self.category_id,
                 'priority': self.priority_id,
                 'case_status': self.confirmed_id,
-                'notes': 'Created by tcms_api.plugin_helpers.Backend',
+                'notes': self.created_by_text,
                 'is_automated': True,
             })]
             created = True
@@ -519,10 +569,13 @@ class Backend:  # pylint: disable=too-many-instance-attributes
             :type comment: str
             :return: None
         """
-        self.rpc.TestExecution.update(test_execution_id, {
+        args = {
             'status': status_id,
-            'tested_by': self.default_tester_id(),
-        })
+        }
+        if self.default_tester_id:
+            args['tested_by'] = self.default_tester_id
+
+        self.rpc.TestExecution.update(test_execution_id, args)
 
         if comment:
             self.add_comment(test_execution_id, comment)
@@ -542,3 +595,13 @@ class Backend:  # pylint: disable=too-many-instance-attributes
             :return: None
         """
         self.rpc.TestExecution.add_comment(test_execution_id, comment)
+
+    @property
+    def created_by_text(self):
+        if not self.name:
+            raise NotImplementedError
+
+        if not self.version:
+            raise NotImplementedError
+
+        return f'Created by {self.name}, version {self.version}'
